% EXAMPLE.M
%
% Empirical example of rank estimation for FCI systems
% 	in Nielsen and Shimotsu (2006).
%
%	The exchange data is collected by Morten Nielsen.
%
%								Katsumi Shimotsu, February 2007
%_______________________________________________________________


clear;

data = wk1read('exrate.wk1');
data(1,:) = [];
data(:,1:8) = [];
[n,dum] = size(data);

y = ['LCAN'
'LSWE'
'LFRA'
'LGER'
'LITA'
'LJPN'
'LUK '];

alpha1 = 0.6;
alpha2 = 0.5;

m11 = fix(n^alpha1);
m12 = fix(n^alpha2);
std1 = 1/2/sqrt(m11)*1.96;
std2 = 1/2/sqrt(m12)*1.96;

fprintf(1,'Number of observations : %6i  \n',n);
fprintf(1,'m11 = n^0.6            : %6i\n',m11);
fprintf(1,'standard error estimate: %6.4f\n\n',std1); % The number in the paper
% needs to be multiplied by 1.96.
fprintf(1,'m12 = n^0.5            : %6i  \n',m12);
fprintf(1,'standard error estimate: %6.4f\n',std2);

% estiamte d of each series

for i=1:7
	
	x = data(:,i);

	% estimate d of series i
	disp(y(i,:));
	ew1 = felw2st(x,m11,3,10);
	ew2 = felw2st(x,m12,3,10);
	fprintf(1,'dhat with m11: %6.4f \n',ew1);
	fprintf(1,'dhat with m12: %6.4f \n\n',ew2);

end


h = 1/log(n);			% In the paper, it is written as (1/log(n))^(1/2) by
						% mistake, but this is the correct bandwidth used.
h2 = 1/log(n)/log(n);	% In the paper, it is written as (1/log(n)) by mistake.

m1 = fix(n^0.6);
m = fix(n^0.55);
v = m^(-0.45);
[dhat,ghat,ehat,pehat,pihat,shat,l,rhat] = rankest(data,m1,m,v,1);
fprintf(1,'The estimates of d with m1 = n^0.6 \n');
disp(dhat);
fprintf(1,'The estimates of 10000*eig(G) with m = n^0.55 \n');
disp(fliplr(ehat)*10000);
fprintf(1,'The estimates of eig(P) with m = n^0.55 \n');
disp(pehat);

t1 = t0stat(dhat,ghat,m1,h);	% T0 statistic
t2 = t0stat(dhat,ghat,m1,h2);	% T0 statistic


for k = 0.45:-0.1:0.05
    v = m^(-k);
    [dhat,ghat,ehat,pehat,pihat,shat,l,rhat] = rankest(data,m1,m,v,1);
    fprintf(1,'v = m^%5.3f \n\n',-k);
    fprintf(1,'l(u), u=0,...,6\n');
    disp(l);
    fprintf(1,'rhat=0 rhat=1 rhat=2 rhat=3 rhat=4 rhat=5 rhat=6\n');
    disp(rhat);
end

m1 = fix(n^0.5);
m = fix(n^0.45);
v = m^(-0.45);
[dhat,ghat,ehat,pehat,pihat,shat,l,rhat] = rankest(data,m1,m,v,1);
fprintf(1,'The estimates of d with m1 = n^0.5 \n');
disp(dhat);
fprintf(1,'The estimates of 10000*eig(G) with m = n^0.45 \n');
disp(fliplr(ehat)*10000);
fprintf(1,'The estimates of eig(P) with m = n^0.45 \n');
disp(pehat);

t3 = t0stat(dhat,ghat,m1,h);	% T0 statistic
t4 = t0stat(dhat,ghat,m1,h2);	% T0 statistic

for k = 0.45:-0.1:0.05
    v = m^(-k);
    [dhat,ghat2,ehat,pehat,pihat,shat,l,rhat] = rankest(data,m1,m,v,1);
    fprintf(1,'v = m^%5.3f \n\n',-k);
    fprintf(1,'l(u), u=0,...,6\n');
    disp(l);
    fprintf(1,'rhat=0 rhat=1 rhat=2 rhat=3 rhat=4 rhat=5 rhat=6\n');
    disp(rhat);
end
