function[dhat,ghat,ehat,pehat,pihat,shat,l,rhat] = rankest(x,m1,m,v,param);
% function[ghat,ehat,pihat,shat,l,rhat] = rankest(x,m1,m);
% RANKEST.M computes various rank estimates
% following Nielsen and Shimotsu (2004) and Robinson and Yajima (2002)
%
%										   Katsumi Shimotsu, August 2004
%
%		Unknown mean is corrected by the method of Shimotsu (2004)
%
%		Dependency: this code requires fracdiff.m, felw2st.m, ewhittle.m
%
% 		INPUT	x: data (n*p vector)
%				m1: truncation number for estimating d
%				m: truncation number for estimating G
%               v: v(n) in the model selection procedure
%               param: 0 if no mean correction, 1 if mean correction
%
%       OUTPUT  ghat: p*p
%               dhat,ehat,pehat,l,rhat: 1*p
%               pihat,shat: 1*(p-1)
%
%____________________________________________________________________________

[n,p] = size(x);

t = (0:1:n-1)';
lambda = 2*pi*t/n;      % Fourier frequencies

dhat = zeros(1,p);
dx = zeros(n,p);
        
    for k = 1:p
    
    if param ==1    

        dhat(k) = felw2st(x(:,k),m1,3,10);
        d = dhat(k);
        weight = (d<=0.5) + (1/2)*(1 + cos(-2*pi+4*pi*d))*(d>0.5)*(d<0.75);
		xbar = sum(x(:,k))/n;
		myu = weight*xbar + (1-weight)*x(1,k);
		x(:,k) = x(:,k) - myu;
        
    else
        
        dhat(k) = fminbnd('ewhittle',-1,3,[],x(:,k),m1,0);    
        
    end
    
        dx(:,k) = fracdiff(x(:,k),dhat(k));
        
    end

wdx = (2*pi*n)^(-1/2)*conj(fft(conj(dx))).*repmat(exp(i*lambda),1,p);
wdx = wdx(2:m+1,:);
wdx = conj(wdx);
ghat = real(wdx'*wdx/m);
        
ehat = sort(eig(ghat));   % note that the smallest is the first element

pimat1 = triu(toeplitz(ehat(1:p-1)));
pihat = sum(pimat1)/sum(ehat);
ehat2 = flipud(ehat);   % now the largest is the first element
pimat2 = tril(toeplitz(ehat2(1:p-1)));
pimat12 = pimat1.^2; 
pimat22 = pimat2.^2; 
shat = (sum(pimat1).^2).*(sum(pimat22)) + (sum(pimat2).^2).*(sum(pimat12));
shat = shat/(sum(ehat))^4;

% construct the correlation matrix (Robinson and Yajima, p 231)

dmat = diag(diag(ghat).^-0.5);      % D matrix
pmat = dmat*ghat*dmat;
pehat = sort(eig(pmat));    % note that the smallest is the first element
pehat = flipud(pehat);      % now the largest is the first element

l = v*(fliplr(1:p)) - flipud(cumsum(pehat))';   % vector of l(u), u=0,...,p-1
rhat = (l == min(l));                           % rhat

ehat = ehat';       % transform ehat to a row vector
pehat = pehat';       % transform pehat to a row vector

%ehat3(p) = [];
%l = v*(fliplr(1:p-1)) - flipud(cumsum(ehat3))'; % vector of l(u), u=1,...,p-1
%rhat = (l == min(l));                           % rhat

%l3 = v*(fliplr(1:p-1)) - sum(pimat2);   % l(u) from Ghat, for reference
%rhat3 = (l3 == min(l3));
